package ink.wgink.module.file.media.controller.api.rtmp;

import ink.wgink.annotation.CheckRequestBodyAnnotation;
import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import ink.wgink.module.file.media.pojo.dtos.rtmp.RtmpClientDTO;
import ink.wgink.module.file.media.pojo.vos.rtmp.RtmpClientVO;
import ink.wgink.module.file.media.service.rtmp.IRtmpClientService;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.Arrays;
import java.util.List;
import java.util.Map;

/**
 * @ClassName: ClientController
 * @Description: rtmp客户端
 * @Author: CodeFactory
 * @Date: 2022-04-26 22:03:44
 * @Version: 3.0
 **/
@Api(tags = ISystemConstant.API_TAGS_SYSTEM_PREFIX + "rtmp客户端接口")
@RestController
@RequestMapping(ISystemConstant.API_PREFIX + "/rtmp-client")
public class RtmpClientController extends DefaultBaseController {

    @Autowired
    private IRtmpClientService clientService;

    @ApiOperation(value = "新增rtmp客户端", notes = "新增rtmp客户端接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PostMapping("save")
    @CheckRequestBodyAnnotation
    public SuccessResult save(@RequestBody RtmpClientVO rtmpClientVO) {
        clientService.save(rtmpClientVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "删除rtmp客户端", notes = "删除rtmp客户端接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "ids", value = "ID列表，用下划线分隔", paramType = "path", example = "1_2_3")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @DeleteMapping("remove/{ids}")
    public SuccessResult remove(@PathVariable("ids") String ids) {
        clientService.remove(Arrays.asList(ids.split("\\_")));
        return new SuccessResult();
    }

    @ApiOperation(value = "修改rtmp客户端", notes = "修改rtmp客户端接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "clientId", value = "rtmp客户端ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @PutMapping("update/{clientId}")
    @CheckRequestBodyAnnotation
    public SuccessResult update(@PathVariable("clientId") String clientId, @RequestBody RtmpClientVO rtmpClientVO) {
        clientService.update(clientId, rtmpClientVO);
        return new SuccessResult();
    }

    @ApiOperation(value = "rtmp客户端详情", notes = "rtmp客户端详情接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "clientId", value = "rtmp客户端ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{clientId}")
        public RtmpClientDTO get(@PathVariable("clientId") String clientId) {
        return clientService.get(clientId);
    }

    @ApiOperation(value = "rtmp客户端列表", notes = "rtmp客户端列表接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<RtmpClientDTO> list() {
        Map<String, Object> params = requestParams();
        return clientService.list(params);
    }

    @ApiOperation(value = "rtmp客户端分页列表", notes = "rtmp客户端分页列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
        @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
        @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<RtmpClientDTO>> listPage(ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return clientService.listPage(page);
    }

    @ApiOperation(value = "rtmp客户端统计", notes = "rtmp客户端统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("count")
    SuccessResultData<Integer> count() {
        Map<String, Object> params = requestParams();
        return new SuccessResultData<>(clientService.count(params));
    }

}