package ink.wgink.module.file.media.controller.app.api.rtmp;

import ink.wgink.common.base.DefaultBaseController;
import ink.wgink.interfaces.consts.ISystemConstant;
import ink.wgink.module.file.media.pojo.dtos.rtmp.RtmpClientDTO;
import ink.wgink.module.file.media.service.rtmp.IRtmpClientService;
import ink.wgink.pojo.ListPage;
import ink.wgink.pojo.result.ErrorResult;
import ink.wgink.pojo.result.SuccessResultData;
import ink.wgink.pojo.result.SuccessResultList;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: ClientAppController
 * @Description: rtmp客户端
 * @Author: CodeFactory
 * @Date: 2022-04-26 22:03:44
 * @Version: 3.0
 **/
@Api(tags = ISystemConstant.API_TAGS_APP_PREFIX + "rtmp客户端接口")
@RestController
@RequestMapping(ISystemConstant.APP_PREFIX + "/rtmp-client")
public class RtmpClientAppController extends DefaultBaseController {

    @Autowired
    private IRtmpClientService clientService;

    @ApiOperation(value = "rtmp客户端详情(通过ID)", notes = "rtmp客户端详情(通过ID)接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "clientId", value = "rtmp客户端ID", paramType = "path")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("get/{clientId}")
    public RtmpClientDTO get(@RequestHeader("token") String token, @PathVariable("clientId") String clientId) {
        return clientService.get(clientId);
    }

    @ApiOperation(value = "rtmp客户端列表", notes = "rtmp客户端列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("list")
    public List<RtmpClientDTO> list(@RequestHeader("token") String token) {
        Map<String, Object> params = requestParams();
        return clientService.list(params);
    }

    @ApiOperation(value = "rtmp客户端分页列表", notes = "rtmp客户端分页列表接口")
    @ApiImplicitParams({
        @ApiImplicitParam(name = "token", value = "token", paramType = "header"),
        @ApiImplicitParam(name = "page", value = "当前页码", paramType = "query", dataType = "int", defaultValue = "1"),
        @ApiImplicitParam(name = "rows", value = "显示数量", paramType = "query", dataType = "int", defaultValue = "20"),
        @ApiImplicitParam(name = "keywords", value = "关键字", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "startTime", value = "开始时间", paramType = "query", dataType = "String"),
        @ApiImplicitParam(name = "endTime", value = "结束时间", paramType = "query", dataType = "String")
    })
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("listpage")
    public SuccessResultList<List<RtmpClientDTO>> listPage(@RequestHeader("token") String token, ListPage page) {
        Map<String, Object> params = requestParams();
        page.setParams(params);
        return clientService.listPage(page);
    }

    @ApiOperation(value = "rtmp客户端统计", notes = "rtmp客户端统计接口")
    @ApiResponses({@ApiResponse(code = 400, message = "请求失败", response = ErrorResult.class)})
    @GetMapping("count")
    SuccessResultData<Integer> count() {
        Map<String, Object> params = requestParams();
        return new SuccessResultData<>(clientService.count(params));
    }

}