package ink.wgink.module.file.media.dao.rtmp;

import ink.wgink.exceptions.RemoveException;
import ink.wgink.exceptions.SaveException;
import ink.wgink.exceptions.SearchException;
import ink.wgink.exceptions.UpdateException;
import ink.wgink.interfaces.init.IInitBaseTable;
import ink.wgink.module.file.media.pojo.bos.rtmp.RtmpClientBO;
import ink.wgink.module.file.media.pojo.pos.rtmp.RtmpClientPO;
import ink.wgink.module.file.media.pojo.dtos.rtmp.RtmpClientDTO;
import org.springframework.stereotype.Repository;

import java.util.List;
import java.util.Map;

/**
 * @ClassName: IClientDao
 * @Description: rtmp客户端
 * @Author: CodeFactory
 * @Date: 2022-04-26 22:03:44
 * @Version: 3.0
 **/
@Repository
public interface IRtmpClientDao extends IInitBaseTable {

    /**
     * 新增rtmp客户端
     *
     * @param params
     * @throws SaveException
     */
    void save(Map<String, Object> params) throws SaveException;

    /**
     * 删除rtmp客户端
     *
     * @param params
     * @throws RemoveException
     */
    void remove(Map<String, Object> params) throws RemoveException;

    /**
     * 删除rtmp客户端（物理）
     *
     * @param params
     * @throws RemoveException
     */
    void delete(Map<String, Object> params) throws RemoveException;

    /**
     * 修改rtmp客户端
     *
     * @param params
     * @throws UpdateException
     */
    void update(Map<String, Object> params) throws UpdateException;

    /**
     * rtmp客户端详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    RtmpClientDTO get(Map<String, Object> params) throws SearchException;

    /**
     * rtmp客户端详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    RtmpClientBO getBO(Map<String, Object> params) throws SearchException;

    /**
     * rtmp客户端详情
     *
     * @param params
     * @return
     * @throws SearchException
     */
    RtmpClientPO getPO(Map<String, Object> params) throws SearchException;

    /**
     * rtmp客户端列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RtmpClientDTO> list(Map<String, Object> params) throws SearchException;

    /**
     * rtmp客户端列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RtmpClientBO> listBO(Map<String, Object> params) throws SearchException;

    /**
     * rtmp客户端列表
     *
     * @param params
     * @return
     * @throws SearchException
     */
    List<RtmpClientPO> listPO(Map<String, Object> params) throws SearchException;

    /**
     * rtmp客户端统计
     *
     * @param params
     * @return
     * @throws SearchException
     */
    Integer count(Map<String, Object> params) throws SearchException;

    /**
     * 最新的客户端编码
     *
     * @param datetime
     * @return
     * @throws SearchException
     */
    String latestClientCode(String datetime) throws SearchException;

}